@@ -531,6 +531,8 @@ make_client_session(struct client_options *options);
 #define OPT_SUBSYSTEM 0x214
 #define OPT_DETACH 0x215
 
+#define OPT_ASKPASS 0x216
+
 void
 init_client_options(struct client_options *self,
 		    struct randomness *random,
@@ -538,7 +540,7 @@ init_client_options(struct client_options *self,
 		    int *exit_code)			 
 {
   self->random = random;
-  
+
   self->tty = make_unix_interact();
   self->escape = -1;
   
@@ -548,7 +550,7 @@ init_client_options(struct client_options *self,
   
   self->not = 0;
   self->port = NULL;
-  self->remote = NULL;
+  self->target = NULL;
 
   self->local_user = self->user = getenv(ENV_LOGNAME);
 
@@ -577,41 +579,15 @@ init_client_options(struct client_options *self,
   object_queue_init(&self->actions);  
 }
 
-/* Host to connect to */
-DEFINE_COMMAND(client_options2remote)
-     (struct command *s UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST_SUBTYPE(client_options, options, a);
-  trace("client.c: client_options2remote\n");
-  
-  COMMAND_RETURN(c, options->remote);
-}
-
-/* Host to connect to */
-DEFINE_COMMAND(client_options2actions)
-     (struct command *s UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST_SUBTYPE(client_options, options, a);
-
-  trace("client.c: client_options2actions, %i actions\n",
-	options->actions.length);
-  
-  COMMAND_RETURN(c, queue_to_list(&options->actions));
-}
-
 static const struct argp_option
 client_options[] =
 {
   /* Name, key, arg-name, flags, doc, group */
   { "port", 'p', "Port", 0, "Connect to this port.", 0 },
   { "user", 'l', "User name", 0, "Login as this user.", 0 },
-
+  { "askpass", OPT_ASKPASS, "Program", 0,
+    "Program to use for reading passwords. "
+    "Should be an absolute filename.", 0 },
   { NULL, 0, NULL, 0, "Actions:", CLIENT_ARGP_ACTION_GROUP },
   { "forward-local-port", 'L', "local-port:target-host:target-port", 0, "", 0 },
 #if 0
@@ -788,7 +764,7 @@ client_shell_session(struct client_options *options)
 /* Create a session for a subsystem */
 static struct command *
 client_subsystem_session(struct client_options *options,
-		       struct lsh_string *subsystem)
+			 struct lsh_string *subsystem)
 {
   struct ssh_channel *session = make_client_session(options);
   
@@ -938,35 +914,27 @@ fork_output(int out)
     }
 }
 
-/* FIXME: Use const? */
-
-#define DEFINE_CALLBACK(name) \
-static void do_##name(struct lsh_callback *self); \
-static struct lsh_callback \
-name = { STATIC_HEADER, do_##name }; \
-static void do_##name(struct lsh_callback *self UNUSED)
-
 /* A callback that exits the process immediately. */
-DEFINE_CALLBACK(exit_callback)
+DEFINE_ESCAPE(exit_callback, "Exit.")
 {
   exit(EXIT_SUCCESS);
 }
 
-DEFINE_CALLBACK(verbose_callback)
+DEFINE_ESCAPE(verbose_callback, "Toggle verbose messages.")
 {
   verbose_flag = !verbose_flag;
   if (verbose_flag)
     verbose("Enabling verbose messages\n");
 }
 
-DEFINE_CALLBACK(debug_callback)
+DEFINE_ESCAPE(debug_callback, "Toggle debug messages.")
 {
   debug_flag = !debug_flag;
   if (debug_flag)
     debug("Enabling debug messages\n");
 }
 
-DEFINE_CALLBACK(quiet_callback)
+DEFINE_ESCAPE(quiet_callback, "Toggle warning messages.")
 {
   quiet_flag = !quiet_flag;
   if (!quiet_flag)
@@ -1269,18 +1237,8 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
       break;
     case ARGP_KEY_ARG:
       if (!state->arg_num)
-	{
-	  if (options->port)
-	    options->remote = make_address_info_c(arg, options->port, 0);
-	  else
-	    options->remote = make_address_info_c(arg, "ssh", 22);
-	  
-	  if (!options->remote)
-	    argp_error(state, "Invalid port or service '%s'.",
-		       options->port ? options->port : "22");
-	  
-	  break;
-	}
+	options->target = arg;
+      
       else
 	/* Let the next case parse it.  */
 	return ARGP_ERR_UNKNOWN;
@@ -1334,6 +1292,10 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
       options->user = arg;
       break;
 
+    case OPT_ASKPASS:
+      INTERACT_SET_ASKPASS(options->tty, arg);
+      break;
+      
     case 'e':
       if (arg[0] && !arg[1])
 	/* A single char argument */
