@@ -21,6 +21,12 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+
 #include "client.h"
 
 #include "channel_commands.h"
@@ -31,8 +37,6 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-
 #include "client_pty.c.x"
 
 /* GABA:
@@ -61,6 +65,8 @@ do_kill_client_tty_resource(struct resource *s)
   CAST(client_tty_resource, self, s);
   self->super.alive = 0;
   INTERACT_SET_ATTRIBUTES(self->tty, self->attr);
+  /* Tell the werror functions that terminal mode is restored. */
+  set_error_raw(0);
 }
 
 static struct resource *
@@ -147,6 +153,9 @@ do_pty_continuation(struct command_continuation *s,
 	     "Setting the attributes of the local terminal failed.\n");
     }
 
+  /* Tell the werror functions that terminal mode is restored. */
+  set_error_raw(1);
+  
   remember_resource(channel->resources,
 		    make_client_tty_resource(self->req->tty,
 					     self->req->attr));
