@@ -1,6 +1,5 @@
 /* gateway_commands.h
  *
- * $Id: gateway_commands.h,v 1.3 2002/08/25 08:17:09 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -36,7 +35,4 @@ extern struct command_2 gateway_accept;
 struct command *
 make_gateway_setup(struct local_info *local);
 
-extern struct command gateway_setup_command;
-#define GATEWAY_SETUP (&gateway_setup_command.super)
-
 #endif /* GATEWAY_COMMANDS_H_INCLUDED */
