@@ -38,6 +38,10 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "nettle/sexp.h"
+/* For struct spki_iterator */
+#include "spki/parse.h"
+
 #include "algorithms.h"
 #include "alist.h"
 #include "atoms.h"
@@ -69,31 +73,15 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include "nettle/sexp.h"
-/* For struct spki_iterator */
-#include "spki/parse.h"
-
 
 #include "lsh_argp.h"
 
 /* Forward declarations */
-struct command_2 lsh_verifier_command;
-#define OPTIONS2VERIFIER (&lsh_verifier_command.super.super)
-
-struct command_2 lsh_login_command;
-#define LSH_LOGIN (&lsh_login_command.super.super)
-
-struct command options2service;
-#define OPTIONS2SERVICE (&options2service.super)
-
-static struct command options2identities;
-#define OPTIONS2IDENTITIES (&options2identities.super)
 
 static struct request_service request_userauth_service =
 STATIC_REQUEST_SERVICE(ATOM_SSH_USERAUTH);
 #define REQUEST_USERAUTH_SERVICE (&request_userauth_service.super.super)
 
-
 #include "lsh.c.x"
 
 /* Block size for stdout and stderr buffers */
@@ -120,10 +108,6 @@ STATIC_REQUEST_SERVICE(ATOM_SSH_USERAUTH);
        (with_srp_keyexchange . int)
        (with_dh_keyexchange . int)
 
-       ; Command to invoke to start ssh-connection service)
-       (service object command)
-       
-       ;; (kexinit object make_kexinit)
        (kex_algorithms object int_list)
        
        (sloppy . int)
@@ -175,18 +159,6 @@ make_options(struct exception_handler *handler,
 }
 
 
-/* Request ssh-userauth or ssh-connection service, as appropriate,
- * and pass the options as a first argument. */
-DEFINE_COMMAND(options2service)
-     (struct command *s UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST(lsh_options, options, a);
-  COMMAND_RETURN(c, options->service);
-}
-
 /* Open hostkey database. By default, "~/.lsh/host-acls". */
 
 static struct spki_context *
@@ -217,18 +189,17 @@ read_known_hosts(struct lsh_options *options)
       if (fd < 0)
 	{
 	  struct stat sbuf;
+	  struct lsh_string *known_hosts
+	    = ssh_format("%lz/.lsh/known_hosts", options->home);
 
-	  /* FIXME: Don't reuse tmp here. */
-	  lsh_string_free(tmp);
-	  tmp = ssh_format("%lz/.lsh/known_hosts", options->home);
-
-	  if (stat(lsh_get_cstring(tmp), &sbuf) == 0)
+	  if (stat(lsh_get_cstring(known_hosts), &sbuf) == 0)
 	    {
 	      werror("You have an old known-hosts file `%S'.\n"
 		     "To work with lsh-2.0, run the lsh-upgrade script,\n"
 		     "which will convert that to a new host-acls file.\n",
 		     tmp);
 	    }
+	  lsh_string_free(known_hosts);
 	}
     }
 
@@ -288,7 +259,7 @@ read_user_keys(struct lsh_options *options)
   struct verifier *v;
   struct lsh_string *spki_public;
 
-  trace("read-user_keys\n");
+  trace("read_user_keys\n");
   
   if (!options->with_publickey)
     return make_object_list(0, -1);
@@ -395,23 +366,6 @@ read_user_keys(struct lsh_options *options)
     }
 }
 
-/* FIXME: Call read_user_key directly from main instead. */
-
-/* Read user's private keys. By default, "~/.lsh/identity". */
-static void
-do_options2identities(struct command *ignored UNUSED,
-		      struct lsh_object *a,
-		      struct command_continuation *c,
-		      struct exception_handler *e UNUSED)
-{
-  CAST(lsh_options, options, a);
-
-  COMMAND_RETURN(c, read_user_keys(options));
-}
-
-static struct command options2identities =
-STATIC_COMMAND(do_options2identities);
-
 
 /* Maps a host key to a (trusted) verifier object. */
 
@@ -423,7 +377,7 @@ STATIC_COMMAND(do_options2identities);
        (db object spki_context)
        (tty object interact)
        (access string)
-       (host object address_info)
+       (host . "const char *")
        ; Allow unauthorized keys
        (sloppy . int)
        ; If non-null, append an ACL for the received key to this file.
@@ -485,25 +439,6 @@ do_lsh_lookup(struct lookup_verifier *c,
 	break;
       }
       
-#if 0
-    case ATOM_RSA_PKCS1_SHA1_LOCAL:
-      {
-	struct rsa_verifier *v = make_ssh_rsa_verifier(key->length, key->data);
-	if (!v)
-	  {
-	    werror("do_lsh_lookup: Invalid ssh-rsa key.\n");
-	    return NULL;
-	  }
-	subject = SPKI_LOOKUP(self->db,
-			      rsa_to_spki_public_key(&v->public),
-			      &v->super);
-	assert(subject);
-	assert(subject->verifier);
-	break;
-      }
-      
-    case ATOM_SPKI:
-#endif
       /* It doesn't matter here which flavour of SPKI is used. */
     case ATOM_SPKI_SIGN_RSA:
     case ATOM_SPKI_SIGN_DSS:
@@ -577,17 +512,17 @@ do_lsh_lookup(struct lookup_verifier *c,
 	  
 	  if (!INTERACT_YES_OR_NO
 	      (self->tty,
-	       ssh_format("Received unauthenticated key for host %lS\n"
+	       ssh_format("Received unauthenticated key for host %lz\n"
 			  "Key details:\n"
 			  "Bubble Babble: %lfS\n"
 			  "Fingerprint:   %lfS\n"
 			  /* "SPKI SHA1:     %lfxS\n" */
 			  "Do you trust this key? (y/n) ",
-			  self->host->ip, babble, fingerprint /* , spki_fingerprint */), 0, 1))
+			  self->host, babble, fingerprint /* , spki_fingerprint */), 0, 1))
 	    return NULL;
 	}
 
-      acl = lsh_sexp_format(0, "(acl(entry(subject%l%l)))",
+      acl = lsh_sexp_format(0, "(acl(entry(subject%l)%l))",
 			    subject->key_length, subject->key,
 			    self->access->length, self->access->data);
       
@@ -601,7 +536,7 @@ do_lsh_lookup(struct lookup_verifier *c,
       /* Write an ACL to disk. */
       if (self->file)
 	{
-	  A_WRITE(self->file, ssh_format("\n; ACL for host %lS\n", self->host->ip));
+	  A_WRITE(self->file, ssh_format("\n; ACL for host %lz\n", self->host));
 	  A_WRITE(self->file,
 		  lsh_sexp_format(1, "%l", acl->length, acl->data));
 	  lsh_string_free(acl);
@@ -616,7 +551,7 @@ do_lsh_lookup(struct lookup_verifier *c,
 static struct lookup_verifier *
 make_lsh_host_db(struct spki_context *db,
 		 struct interact *tty,
-		 struct address_info *host,
+		 const char *host,
 		 int sloppy,
 		 struct abstract_write *file)
 {
@@ -634,101 +569,53 @@ make_lsh_host_db(struct spki_context *db,
   return &res->super;
 }
 
-/* Takes options and an spki_context as argument and returns a
- * lookup_verifier */
-
-DEFINE_COMMAND2(lsh_verifier_command)
-     (struct command_2 *s UNUSED,
-      struct lsh_object *a1,
-      struct lsh_object *a2,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
+static struct command *
+make_lsh_login(struct lsh_options *options,
+	       struct object_list *keys)
 {
-  CAST(lsh_options, options, a1);
-  CAST_SUBTYPE(spki_context, db, a2);
-  COMMAND_RETURN(c, make_lsh_host_db(db,
-				     options->super.tty,
-				     options->super.remote,
-				     options->sloppy,
-				     options->capture_file));
-}
-
-
-/* (login options public-keys connection) */
-DEFINE_COMMAND2(lsh_login_command)
-     (struct command_2 *s UNUSED,
-      struct lsh_object *a1,
-      struct lsh_object *a2,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST(lsh_options, options, a1);
-  CAST_SUBTYPE(object_list, keys, a2);
-
   struct client_userauth_method *password
     = make_client_password_auth(options->super.tty);
 
   /* FIXME: Perhaps we should try "none" only when using SRP. */
   struct client_userauth_method *none
     = make_client_none_auth();
+
+  struct object_list *methods;
+  if (LIST_LENGTH(keys))
+    methods = make_object_list(3,
+			       none,
+			       make_client_publickey_auth(keys),
+			       password,
+			       -1);
+  else
+    methods = make_object_list(2, none, password, -1);
   
-  COMMAND_RETURN(c,
-		 make_client_userauth
-		 (ssh_format("%lz", options->super.user),
-		  ATOM_SSH_CONNECTION,
-		  (LIST_LENGTH(keys)
-		   ? make_object_list
-		   (3,
-                    none,
-		    make_client_publickey_auth(keys),
-		    password,
-		    -1)
-		   : make_object_list(2, none, password, -1))));
+  return make_client_userauth
+    (ssh_format("%lz", options->super.user),
+     ATOM_SSH_CONNECTION, methods);
 }
 
-
-/* NOTE: options2identities can block for reading, so it must not be
- * invoked directly. */
-
-/* Requests the ssh-userauth service, log in, and request connection
- * service. */
-/* GABA:
-   (expr
-     (name make_lsh_userauth)
-     (params
-       (options object lsh_options))
-     (expr
-       (lambda (connection)
-         (lsh_login options
-	   
-	   ; The prog1 delay is needed because options2identities
-	   ; may not return immediately.
-	   (options2identities (prog1 options connection))
-
-	   ; Request the userauth service
-	   (request_userauth_service connection))))) */
-
 /* GABA:
    (expr
      (name make_lsh_connect)
      (params
        (handshake object handshake_info)
        (init object make_kexinit)
-       (db object lookup_verifier))
-     (expr (lambda (options)
+       (db object lookup_verifier)
+       (actions object object_list)
+       (login object command))
+     (expr (lambda (remote)
                ; What to do with the service
-	       ((progn (options2actions options))
-	         ; Initialize service
+	       ((progn actions)
 		 (init_connection_service
-		   ; Either requests ssh-connection service,
-		   ; or requests and uses the ssh-userauth service.
-		   (options2service options
-		     ; Start the ssh transport protocol
-	             (connection_handshake
-		       handshake init
-		       db
- 		       ; Connect using tcp
-		       (connect_simple (options2remote options)))))))))
+		   (login
+		     (request_userauth_service
+		       ; Start the ssh transport protocol
+		       (connection_handshake
+			 handshake init
+			 db
+			 ; Connect using tcp
+			 (connect_list remote)))))))))
 */
 
 
@@ -901,11 +788,6 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	}
       else
 	argp_error(state, "All keyexchange algorithms disabled.");
-
-      {
-        CAST_SUBTYPE(command, o, make_lsh_userauth(self));
-        self->service = o;
-      }
 	
       {
 	struct lsh_string *tmp = NULL;
@@ -956,7 +838,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	    }
 	  gateway = make_gateway_address(self->super.local_user,
 					 self->super.user,
-					 self->super.remote);
+					 self->super.target);
 
 	  if (!gateway)
 	    {
@@ -1102,6 +984,8 @@ int main(int argc, char **argv, const char** envp)
 {
   struct lsh_options *options;
   struct spki_context *spki;
+  struct object_list *keys;
+  struct connect_list_state *remote;
   
   /* Default exit code if something goes wrong. */
   int lsh_exit_code = 17;
@@ -1133,7 +1017,18 @@ int main(int argc, char **argv, const char** envp)
       return EXIT_FAILURE;
     }
 
+  remote = make_connect_list_state();;
+  
+  if (!io_resolv_address(options->super.target,
+			 options->super.port, 22,
+			 &remote->q))
+    {
+      werror("Could not resolv address `%z'\n", options->super.target);
+      return EXIT_FAILURE;
+    }
+  
   spki = read_known_hosts(options);
+  keys = read_user_keys(options);
   
   {
     struct lsh_object *o =
@@ -1153,15 +1048,16 @@ int main(int argc, char **argv, const char** envp)
 			    make_int_list(0, -1)),
         make_lsh_host_db(spki,
                          options->super.tty,
-                         options->super.remote,
+                         options->super.target,
                          options->sloppy,
-                         options->capture_file));
+                         options->capture_file),
+	queue_to_list(&options->super.actions),
+	make_lsh_login(options, keys));
     
     CAST_SUBTYPE(command, lsh_connect, o);
 
-    COMMAND_CALL(lsh_connect, options, &discard_continuation,
+    COMMAND_CALL(lsh_connect, remote, &discard_continuation,
 		 handler);
-	
   } 
 
 #if 0
