@@ -23,6 +23,30 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+
+#include <errno.h>
+#include <locale.h>
+#include <stdio.h>
+/* #include <string.h> */
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#if TIME_WITH_SYS_TIME && HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#if HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include "algorithms.h"
 #include "alist.h"
 #include "atoms.h"
@@ -59,41 +83,8 @@
 
 #include "lsh_argp.h"
 
-/* Forward declarations */
-struct command options2local;
-#define OPTIONS2LOCAL (&options2local.super)
-
-struct command options2keys;
-#define OPTIONS2KEYS (&options2keys.super)
-
-struct command options2tcp_wrapper;
-#define OPTIONS2TCP_WRAPPER (&options2tcp_wrapper.super)
-
-struct command_2 close_on_sighup;
-#define CLOSE_ON_SIGHUP (&close_on_sighup.super.super)
-
 #include "lshd.c.x"
 
-#include <assert.h>
-
-#include <errno.h>
-#include <locale.h>
-#include <stdio.h>
-/* #include <string.h> */
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#if TIME_WITH_SYS_TIME && HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-#if HAVE_SYS_RESOURCE_H
-#include <sys/resource.h>
-#endif
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 
 /* Option parsing */
 
@@ -153,16 +144,14 @@ const char *argp_program_bug_address = BUG_ADDRESS;
      (name lshd_options)
      (super algorithms_options)
      (vars
-       (e object exception_handler)
-       
        (reaper object reaper)
        (random object randomness)
        
        (signature_algorithms object alist)
-       (interface . "char *")
+       ;; Addresses to bind
+       (local struct addr_queue)
        (port . "char *")
        (hostkey . "char *")
-       (local object address_info)
        (tcp_wrapper_name . "char *")
        (tcp_wrapper_message . "char *")
 
@@ -192,33 +181,9 @@ const char *argp_program_bug_address = BUG_ADDRESS;
        (corefile . int)
        (pid_file . "const char *")
        ; -1 means use pid file iff we're in daemonic mode
-       (use_pid_file . int)
-       ; Resources that should be killed when SIGHUP is received,
-       ; or when the program exits.
-       (resources object resource_list)))
+       (use_pid_file . int)))
 */
 
-static void
-do_exc_lshd_handler(struct exception_handler *s,
-		    const struct exception *e)
-{
-  switch(e->type)
-    {
-    case EXC_RESOLVE:
-    case EXC_RANDOMNESS_LOW_ENTROPY:
-      werror("lshd: %z\n", e->msg);
-      exit(EXIT_FAILURE);
-    default:
-      EXCEPTION_RAISE(s->parent, e);
-    }
-}
-
-static struct exception_handler *
-make_lshd_exception_handler(struct exception_handler *parent,
-			    const char *context)
-{
-  return make_exception_handler(do_exc_lshd_handler, parent, context);
-}
 
 static struct lshd_options *
 make_lshd_options(void)
@@ -227,23 +192,21 @@ make_lshd_options(void)
 
   init_algorithms_options(&self->super, all_symmetric_algorithms());
 
-  self->e = make_lshd_exception_handler(&default_exception_handler,
-					HANDLER_CONTEXT);
   self->reaper = make_reaper();
   self->random = make_system_random();
 
-  self->signature_algorithms = all_signature_algorithms(self->random); /* OK to initialize with NULL */
-
-  self->interface = NULL;
+  /* OK to initialize with NULL */
+  self->signature_algorithms = all_signature_algorithms(self->random);
 
+  addr_queue_init(&self->local);
+  
   /* Default behaviour is to lookup the "ssh" service, and fall back
    * to port 22 if that fails. */
   self->port = NULL;
   
   /* FIXME: this should perhaps use sysconfdir */  
   self->hostkey = "/etc/lsh_host_key";
-  self->local = NULL;
-
+  
   self->with_dh_keyexchange = 1;
   self->with_srp_keyexchange = 0;
 
@@ -276,52 +239,9 @@ make_lshd_options(void)
   self->use_pid_file = -1;
   self->corefile = 0;
 
-  self->resources = make_resource_list();
-  /* Not strictly needed for gc, but makes sure the
-   * resource list is killed properly by gc_final. */
-  gc_global(&self->resources->super);
-
   return self;
 }
 
-/* Port to listen on */
-DEFINE_COMMAND(options2local)
-     (struct command *s UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST(lshd_options, options, a);
-  /* FIXME: Call bind already here? */
-  COMMAND_RETURN(c, options->local);
-}
-
-/* alist of signature algorithms */
-DEFINE_COMMAND(options2signature_algorithms)
-     (struct command *s UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST(lshd_options, options, a);
-  COMMAND_RETURN(c, options->signature_algorithms);
-}
-
-
-/* FIXME: Call read_host_key directly from main instead. */
-DEFINE_COMMAND(options2keys)
-     (struct command *ignored UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST(lshd_options, options, a);
-
-  struct alist *keys = make_alist(0, -1);
-  read_host_key(options->hostkey, options->signature_algorithms, keys);
-  COMMAND_RETURN(c, keys);
-}
-
 /* GABA:
    (class
      (name pid_file_resource)
@@ -357,7 +277,7 @@ make_pid_file_resource(const char *file)
      (name sighup_close_callback)
      (super lsh_callback)
      (vars
-       (resources object resource_list)))
+       (resource object resource)))
 */
 
 static void
@@ -367,7 +287,7 @@ do_sighup_close_callback(struct lsh_callback *s)
   unsigned nfiles;
   
   werror("SIGHUP received.\n");
-  KILL_RESOURCE_LIST(self->resources);
+  KILL_RESOURCE(self->resource);
   
   nfiles = io_nfiles();
 
@@ -377,56 +297,15 @@ do_sighup_close_callback(struct lsh_callback *s)
 }
 
 static struct lsh_callback *
-make_sighup_close_callback(struct lshd_options *options)
+make_sighup_close_callback(struct resource *resource)
 {
   NEW(sighup_close_callback, self);
   self->super.f = do_sighup_close_callback;
-  self->resources = options->resources;
+  self->resource = resource;
 
   return &self->super;
 }
 
-/* (close_on_sighup options file) */
-DEFINE_COMMAND2(close_on_sighup)
-     (struct command_2 *ignored UNUSED,
-      struct lsh_object *a1,
-      struct lsh_object *a2,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST(lshd_options, options, a1);
-  CAST(lsh_fd, fd, a2);
-
-  remember_resource(options->resources, &fd->super);
-
-  COMMAND_RETURN(c, a2);
-}
-
-
-DEFINE_COMMAND(options2tcp_wrapper)
-     (struct command *s UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-#if WITH_TCPWRAPPERS
-  CAST(lshd_options, options, a);
-
-  if (options->tcp_wrapper_name) 
-    COMMAND_RETURN(c, 
-		   make_tcp_wrapper(
-				    make_string(options->tcp_wrapper_name),
-				    options->tcp_wrapper_message ? 
-				    ssh_format("%lz\n", options->tcp_wrapper_message ) :
-				    ssh_format("")
-				    )
-		   ); 
-  else
-#endif /* WITH_TCPWRAPPERS */
-    COMMAND_RETURN(c, &io_log_peer_command);
-}
-
-
 static const struct argp_option
 main_options[] =
 {
@@ -580,6 +459,50 @@ parse_subsystem_list(char *arg)
   return subsystems;
 }
 
+/* NOTE: On success, modifies interface destructively. */
+static int
+parse_interface(char *interface, const char **host, const char **port)
+{
+  *port = NULL;
+  
+  if (interface[0] == '[')
+    {
+      /* A literal address */
+      char *end;
+      interface++;
+      
+      end = strchr(interface, ']');
+      if (!end)
+	return 0;
+
+      switch (end[1])
+	{
+	case ':':
+	  *port = end + 2;
+	  break;
+	case 0:
+	  break;
+	default:
+	  return 0;
+	}
+
+      *host = interface;
+      *end = 0;
+      return 1;
+    }
+  else
+    {
+      char *end = strchr(interface, ':');
+      if (end)
+	{
+	  *port = end + 1;
+	  *end = 0;
+	}
+      *host = interface;
+      return 1;
+    }
+}
+
 static error_t
 main_argp_parser(int key, char *arg, struct argp_state *state)
 {
@@ -635,16 +558,19 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	else
 	  argp_error(state, "All keyexchange algorithms disabled.");
 
-	if (self->port)
-	  self->local = make_address_info_c(self->interface, self->port, 0);
-	else
-	  self->local = make_address_info_c(self->interface, "ssh", 22);
-      
-	if (!self->local)
-	  argp_error(state, "Invalid interface, port or service, %s:%s'.",
-		     self->interface ? self->interface : "ANY",
-		     self->port);
+	if (addr_queue_is_empty(&self->local))
+	  {
+	    /* Default interface */
 
+	    if (!(self->port
+		  ? io_resolv_address(NULL, self->port, 0, &self->local)
+		  : io_resolv_address(NULL, "ssh", 22, &self->local)))
+		
+	      argp_failure(state, EXIT_FAILURE, 0,
+			   "Strange. Could not resolve the ANY address.");
+	  }
+	assert(!addr_queue_is_empty(&self->local));
+	
 	if (self->use_pid_file < 0)
 	  self->use_pid_file = self->daemonic;
 
@@ -693,6 +619,8 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	break;
       }
     case 'p':
+      /* FIXME: Interpret multiple -p:s as a request to listen on
+       * several ports. */
       self->port = arg;
       break;
 
@@ -701,7 +629,25 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       break;
 
     case OPT_INTERFACE:
-      self->interface = arg;
+      {
+	const char *host;
+	const char *port;
+
+	/* On success, modifies arg destructively. */
+	if (!parse_interface(arg, &host, &port))
+	  argp_error(state, "Invalid interface, port or service: %s.", arg);
+
+	if (!port)
+	  port = self->port;
+	
+	if (!(port
+	      ? io_resolv_address(host, port, 0, &self->local)
+	      : io_resolv_address(host, "ssh", 22, &self->local)))
+	  argp_failure(state, EXIT_FAILURE, 0,
+		       "Address %s:%s could not be resolved.\n",
+		       host, port ? port : "ssh");
+      }
+	
       break;
 
 #if WITH_SSH1_FALLBACK
@@ -851,43 +797,6 @@ main_argp =
 };
 
 
-/* GABA:
-   (expr
-     (name make_lshd_listen)
-     (params
-       (handshake object handshake_info)
-       (init object make_kexinit)
-       (services object command) )
-     (expr (lambda (options)
-             (let ((keys (options2keys options)))
-	       (close_on_sighup options
-	         (listen
-	           (lambda (lv)
-    	             (services (connection_handshake
-    	           		  handshake
-    	           		  (kexinit_filter init keys)
-    	           		  keys 
-				  (options2tcp_wrapper options lv))))
-	           (bind (options2local options)) ))))))
-*/
-
-
-/* Invoked when starting the ssh-connection service */
-/* GABA:
-   (expr
-     (name make_lshd_connection_service)
-     (params
-       (hooks object object_list))
-     (expr
-       (lambda (connection)
-         ((progn hooks)
-	    ; We have to initialize the connection
-	    ; before adding handlers.
-	    (init_connection_service
-	      ; Disconnect if connection->user is NULL
-	      (connection_require_userauth connection)))))))
-*/
-
 static void
 do_terminate_callback(struct lsh_callback *s UNUSED)
 {
@@ -905,11 +814,189 @@ static struct lsh_callback
 sigterm_handler = { STATIC_HEADER, do_terminate_callback };
 
 static void
-install_signal_handlers(struct lshd_options *options)
+install_signal_handlers(struct resource *resource)
 {
   io_signal_handler(SIGTERM, &sigterm_handler);
   io_signal_handler(SIGHUP,
-		    make_sighup_close_callback(options));
+		    make_sighup_close_callback(resource));
+}
+
+static void
+do_exc_lshd_handler(struct exception_handler *self,
+		    const struct exception *e)
+{
+  if (e->type & EXC_IO)
+    {
+      CAST_SUBTYPE(io_exception, exc, e);
+      
+      werror("%z, (errno = %i)\n", e->msg, exc->error);
+    }
+  else
+    EXCEPTION_RAISE(self->parent, e);
+}
+
+static struct exception_handler lshd_exception_handler
+= STATIC_EXCEPTION_HANDLER(do_exc_lshd_handler, &default_exception_handler);
+
+/* Invoked when starting the ssh-connection service */
+/* GABA:
+   (expr
+     (name lshd_connection_service)
+     (params
+       (hooks object object_list))
+     (expr
+       (lambda (connection)
+         ((progn hooks)
+	    ; We have to initialize the connection
+	    ; before adding handlers.
+	    (init_connection_service
+	      ; Disconnect if connection->user is NULL
+	      (connection_require_userauth connection)))))))
+*/
+
+static struct command *
+make_lshd_connection_service(struct lshd_options *options)
+{
+  /* Commands to be invoked on the connection */
+  /* FIXME: Use a queue instead. */
+  struct object_list *connection_hooks;
+  struct command *session_setup;
+    
+  /* Supported channel requests */
+  struct alist *supported_channel_requests
+    = make_alist(2,
+		 ATOM_SHELL, &shell_request_handler,
+		 ATOM_EXEC, &exec_request_handler,
+		 -1);
+    
+#if WITH_PTY_SUPPORT
+  if (options->with_pty)
+    {
+      ALIST_SET(supported_channel_requests,
+		ATOM_PTY_REQ, &pty_request_handler.super);
+      ALIST_SET(supported_channel_requests,
+		ATOM_WINDOW_CHANGE, &window_change_request_handler.super);
+    }
+#endif /* WITH_PTY_SUPPORT */
+
+#if WITH_X11_FORWARD
+  if (options->with_x11_forward)
+    ALIST_SET(supported_channel_requests,
+	      ATOM_X11_REQ, &x11_req_handler.super);
+#endif /* WITH_X11_FORWARD */
+  
+  if (options->subsystems)
+    ALIST_SET(supported_channel_requests,
+	      ATOM_SUBSYSTEM,
+	      &make_subsystem_handler(options->subsystems)->super);
+  
+  session_setup = make_install_fix_channel_open_handler
+    (ATOM_SESSION, make_open_session(supported_channel_requests));
+  
+#if WITH_TCP_FORWARD
+  if (options->with_tcpip_forward)
+    connection_hooks = make_object_list
+      (4,
+       session_setup,
+       make_tcpip_forward_hook(),
+       make_install_fix_global_request_handler
+       (ATOM_CANCEL_TCPIP_FORWARD, &tcpip_cancel_forward),
+       make_direct_tcpip_hook(),
+       -1);
+  else
+#endif
+    connection_hooks
+      = make_object_list (1, session_setup, -1);
+
+  {
+    CAST_SUBTYPE(command, connection_service,
+		 lshd_connection_service(connection_hooks));
+    return connection_service;
+  }
+}
+
+
+/* GABA:
+   (expr
+     (name lshd_listen_callback)
+     (params
+       (handshake object handshake_info)
+       (kexinit object make_kexinit)
+       (keys object alist)
+       (logger object command)
+       (services object command) )
+     (expr (lambda (lv)
+    	      (services (connection_handshake
+	                   handshake
+			   kexinit
+			   keys
+			   (logger lv))))))
+*/
+
+static struct io_callback *
+make_lshd_listen_callback(struct lshd_options *options,
+			  struct alist *keys,
+			  struct command *service)
+{
+  struct int_list *hostkey_algorithms;
+  struct make_kexinit *kexinit;
+  struct command *logger;
+  
+  /* Include only hostkey algorithms that we have keys for. */
+  hostkey_algorithms
+    = filter_algorithms(keys,
+			options->super.hostkey_algorithms);
+  
+  if (!hostkey_algorithms)
+    {
+      werror("No hostkey algorithms advertised.\n");
+      hostkey_algorithms = make_int_list(1, ATOM_NONE, -1);
+    }
+  
+  kexinit = make_simple_kexinit(options->random,
+				options->kex_algorithms,
+				hostkey_algorithms,
+				options->super.crypto_algorithms,
+				options->super.mac_algorithms,
+				options->super.compression_algorithms,
+				make_int_list(0, -1));
+
+#if WITH_TCPWRAPPERS
+  if (options->tcp_wrapper_name)
+    logger = make_tcp_wrapper
+      (make_string(options->tcp_wrapper_name),
+       make_string(options->tcp_wrapper_message
+		   ? options->tcp_wrapper_message : ""));
+  else
+#endif /* WITH_TCPWRAPPERS */
+    logger = &io_log_peer_command;
+  
+  {
+    CAST_SUBTYPE(command, server_callback,
+		 lshd_listen_callback
+		 (make_handshake_info(CONNECTION_SERVER,
+					"lsh - a free ssh",
+					NULL,
+					SSH_MAX_PACKET,
+					options->random,
+					options->super.algorithms,
+					options->sshd1),
+		  kexinit,
+		  keys,
+		  logger,
+		  make_offer_service
+		  (make_alist
+		   (1,
+		    ATOM_SSH_USERAUTH,
+		    make_userauth_service(options->userauth_methods,
+					  options->userauth_algorithms,
+					  make_alist(1, ATOM_SSH_CONNECTION,
+						     service,-1)),
+		    -1))));
+
+    /* There should be no exceptions raised on the fd:s we listen on */
+    return make_listen_callback(server_callback, &lshd_exception_handler);
+  }
 }
 
 int
@@ -917,6 +1004,15 @@ main(int argc, char **argv)
 {
   struct lshd_options *options;
 
+  /* Resources that should be killed when SIGHUP is received,
+   * or when the program exits. */
+  struct resource_list *resources = make_resource_list();
+
+  /* Hostkeys */
+  struct alist *keys = make_alist(0, -1);
+
+  struct resource *fds;
+  
 #if HAVE_SETRLIMIT && HAVE_SYS_RESOURCE_H
   /* Try to increase max number of open files, ignore any error */
 
@@ -928,6 +1024,10 @@ main(int argc, char **argv)
   setrlimit(RLIMIT_NOFILE, &r);
 #endif
 
+  /* Not strictly needed for gc, but makes sure the
+   * resource list is killed properly by gc_final. */
+  gc_global(&resources->super);
+  
   io_init();
   
   /* For filtering messages. Could perhaps also be used when converting
@@ -937,18 +1037,23 @@ main(int argc, char **argv)
   /* FIXME: Choose character set depending on the locale */
   set_local_charset(CHARSET_LATIN1);
 
+  install_signal_handlers(&resources->super);
 
   options = make_lshd_options();
 
-  if (!options)
-    return EXIT_FAILURE;
-
-  install_signal_handlers(options);
-  
   trace("Parsing options...\n");
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
   trace("Parsing options... done\n");  
 
+  if (options->daemonic && !options->no_syslog)
+    {
+#if HAVE_SYSLOG
+      set_error_syslog("lshd");
+#else /* !HAVE_SYSLOG */
+      werror("lshd: No syslog. Further messages will be directed to /dev/null.\n");
+#endif /* !HAVE_SYSLOG */
+    }
+  
   if (!options->corefile && !daemon_disable_core())
     {
       werror("Disabling of core dumps failed.\n");
@@ -960,6 +1065,22 @@ main(int argc, char **argv)
       werror("Failed to initialize randomness generator.\n");
       return EXIT_FAILURE;
     }
+
+  if (!read_host_key(options->hostkey, options->signature_algorithms, keys))
+    return EXIT_FAILURE;
+
+  fds = io_listen_list(&options->local,
+		       make_lshd_listen_callback
+		       (options, keys,
+			make_lshd_connection_service(options)),
+		       &lshd_exception_handler);
+  if (!fds)
+    {
+      werror("Could not bind any address.\n");
+      return EXIT_FAILURE;
+    }
+
+  remember_resource(resources, fds);
   
   if (options->daemonic)
     {
@@ -988,12 +1109,6 @@ main(int argc, char **argv)
         }
       else
         {
-#if HAVE_SYSLOG
-          set_error_syslog("lshd");
-#else /* !HAVE_SYSLOG */
-          werror("lshd: No syslog. Further messages will be directed to /dev/null.\n");
-#endif /* !HAVE_SYSLOG */
-
           switch (daemon_init())
             {
             case 0:
@@ -1014,7 +1129,7 @@ main(int argc, char **argv)
   if (options->use_pid_file)
     {
       if (daemon_pidfile(options->pid_file))
-	remember_resource(options->resources, 
+	remember_resource(resources, 
 			  make_pid_file_resource(options->pid_file));
       else
 	{
@@ -1022,100 +1137,6 @@ main(int argc, char **argv)
 	  return EXIT_FAILURE;
 	}
     }
-  {
-    /* Commands to be invoked on the connection */
-    /* FIXME: Use a queue instead. */
-    struct object_list *connection_hooks;
-    struct command *session_setup;
-    
-    /* Supported channel requests */
-    struct alist *supported_channel_requests
-      = make_alist(2,
-		   ATOM_SHELL, &shell_request_handler,
-		   ATOM_EXEC, &exec_request_handler,
-		   -1);
-    
-#if WITH_PTY_SUPPORT
-    if (options->with_pty)
-      {
-        ALIST_SET(supported_channel_requests,
-                  ATOM_PTY_REQ, &pty_request_handler.super);
-        ALIST_SET(supported_channel_requests,
-                  ATOM_WINDOW_CHANGE, &window_change_request_handler.super);
-      }
-#endif /* WITH_PTY_SUPPORT */
-
-#if WITH_X11_FORWARD
-      if (options->with_x11_forward)
-        ALIST_SET(supported_channel_requests,
-		  ATOM_X11_REQ, &x11_req_handler.super);
-#endif /* WITH_X11_FORWARD */
-
-    if (options->subsystems)
-      ALIST_SET(supported_channel_requests,
-		ATOM_SUBSYSTEM,
-		&make_subsystem_handler(options->subsystems)->super);
-		
-    session_setup = make_install_fix_channel_open_handler
-      (ATOM_SESSION, make_open_session(supported_channel_requests));
-    
-#if WITH_TCP_FORWARD
-    if (options->with_tcpip_forward)
-      connection_hooks = make_object_list
-	(4,
-	 session_setup,
-	 make_tcpip_forward_hook(),
-	 make_install_fix_global_request_handler
-	 (ATOM_CANCEL_TCPIP_FORWARD, &tcpip_cancel_forward),
-	 make_direct_tcpip_hook(),
-	 -1);
-    else
-#endif
-      connection_hooks
-	= make_object_list (1, session_setup, -1);
-    {
-      CAST_SUBTYPE(command, connection_service,
-		   make_lshd_connection_service(connection_hooks));
-      CAST_SUBTYPE(command, server_listen, 		   
-		   make_lshd_listen
-		   (make_handshake_info(CONNECTION_SERVER,
-					"lsh - a free ssh",
-					NULL,
-					SSH_MAX_PACKET,
-					options->random,
-					options->super.algorithms,
-					options->sshd1),
-		    make_simple_kexinit
-		    (options->random,
-		     options->kex_algorithms,
-		     options->super.hostkey_algorithms,
-		     options->super.crypto_algorithms,
-		     options->super.mac_algorithms,
-		     options->super.compression_algorithms,
-		     make_int_list(0, -1)),
-		    make_offer_service
-		    (make_alist
-		     (1,
-		      ATOM_SSH_USERAUTH,
-		      make_userauth_service(options->userauth_methods,
-					    options->userauth_algorithms,
-					    make_alist(1, ATOM_SSH_CONNECTION,
-						       connection_service,-1)),
-		      -1))));
-
-      static const struct report_exception_info report =
-	STATIC_REPORT_EXCEPTION_INFO(EXC_IO, EXC_IO,
-				     "lshd: ");
-	    
-      
-      COMMAND_CALL(server_listen, options,
-		   &discard_continuation,
-		   make_report_exception_handler
-		   (&report,
-		    options->e,
-		    HANDLER_CONTEXT));
-    }
-  }
   
   io_run();
 
