@@ -23,6 +23,12 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+
 #include "server_authorization.h"
 
 #include "format.h"
@@ -32,8 +38,6 @@
 #include "spki.h"
 #include "xalloc.h"
 
-#include <assert.h>
-
 #include "server_authorization.c.x"
 
 /* For now a key is authorized if a file named as the hash of the
